<?php
namespace Teknisa\CADFIS\Services;

use Teknisa\CADFIS\Factories\Ger00200EmpresaFactory;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\Utilities;


class Ger00200EmpresaService{
    
    const DEV_LINK  = 'http://midia.teknisa.com/files/';
    const PROD_LINK = 'https://midia.teknisa.com/files/';
    const EXTENSION_PFX = 'pfx';
    const EXTENSION_P12 = 'p12';
    const EXTENSION_PNG = 'png';
    const EXTENSION_JPG = 'jpg';

    protected $environment;
    protected $entityManager;
    protected $dataSourceManager;
    protected $empresaFactory;

    public function __construct(Ger00200EmpresaFactory $empresaFactory, Environment $environment, \Doctrine\ORM\EntityManager $entityManager ,$dataSourceManager){
        $this->empresaFactory   = $empresaFactory;
        $this->environment      = $environment;
        $this->entityManager    = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
    }

    public function buscar($condicao){
        return $this->dataSourceManager->findBy($condicao);
    }

    public function saveInscricao ($dataSet){
        try {
            $this->dataSourceManager->persist($dataSet);
            $this->entityManager->flush();
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    const GET_INSCRICAO = "SELECT * FROM INSCRESTAD
                            WHERE CDEMPRESA = :CDEMPRESA
                            AND CDINSCESTA = :CDINSCESTA
                            AND NRORG = :NRORG";

    public function consultaInscricao($row){
        try {
            $params = array(
                'CDEMPRESA' => $row[0]['CDEMPRESA'],
                'CDINSCESTA' => $row[0]['CDINSCESTA'],
                'NRORG' => $this->environment->getNrOrgTrab()
            );
            return $this->entityManager->getConnection()->fetchAll(self::GET_INSCRICAO, $params);
        } catch (\Exception $e) {
            throw $e; 
        }
    }

    public  function criaDataSetInscricaoEstadual ($row){
        if($row[0]['__is_new']){
            if($row[0]['CDSENHACERTIF']){
                $rows = $this->criptografarSenhaCertificado($row);
                $dataSet = new DataSet('inscrestad', $rows);
            }else{
                $dataSet = new DataSet('inscrestad', $row);
            }
            return $dataSet;
        }else{

            $resultadoBanco = $this->consultaInscricao($row);
            $inscricao = $resultadoBanco;
            
            if($row[0]['CDSENHACERTIF'] !== $inscricao[0]['CDSENHACERTIF']){
                if($row[0]['CDSENHACERTIF']){
                    $rows = $this->criptografarSenhaCertificado($row);
                    $dataSet = new DataSet('inscrestad', $rows);
                }else{
                    $dataSet = new DataSet('inscrestad', $row);
                }
            }else{
                $dataSet = new DataSet('inscrestad', $row);
            }
            return $dataSet;
        }
    }

    public function criptografarSenhaCertificado($row) {
        try {
            $senhaCriptografada = $this->encrypt($row[0]['CDSENHACERTIF'], 'teknisa');
            $row[0]['CDSENHACERTIF'] = $senhaCriptografada;
            return $row;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getInscrEstad($cdinscesta, $cdempresa, $nrorg, $getLink = false) {
        $inscrEsta = $this->empresaFactory->getInscrEstad($cdinscesta, $cdempresa, $nrorg);
        if(!empty($inscrEsta) && isset($inscrEsta[0]['DSDIRCERTIFICADO'])) {
            $inscrEsta = $inscrEsta[0];
            $inscrEsta['DSDIRCERTARRAY'] = $this->getArrayFiles($inscrEsta['DSDIRCERTIFICADO'], $getLink, 'prod');
        }
        return $inscrEsta;
    }

    public function gravaCertificado ($rowCertificado){
        try {
            $cdinscesta = $rowCertificado['CDINSCESTA'];
            $certificados = $rowCertificado['DSDIRCERTIFICADO'];
            $empresa = $rowCertificado['CDEMPRESA'];
            $nrorg = isset($rowCertificado['NRORG']) ? $rowCertificado['NRORG'] : $this->environment->getNrorg();
            
            $inscrEsta = $this->getInscrEstad($cdinscesta, $empresa, $nrorg);
            
            $dsDirCert    =  '';
            $arrayCert    =  array();

            foreach ($certificados as $file) {
                $fileName  = $file['name'];
                $extension = explode(".", $fileName);
                $extension = end($extension);
               
                if(($extension !== self::EXTENSION_PFX) && ($extension !== self::EXTENSION_P12)) {
                    throw \Exception($fileName.': extensão do arquivo não aceita.');
                }
                else {
                    $dsDirCert.='<'.$file['name'].'>';
                    $dsDirCert.=$file["path"];
                    $this->updateDsDirCert($cdinscesta, $empresa, $nrorg, $dsDirCert);
                }
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function encrypt($text, $salt = 'teknisa') {
        return Utilities::zeedhiEncrypt($text, $salt);
    }
    
    public function updateDsDirCert($cdinscesta, $cdempresa, $nrorg, $dsdircert) {
        $this->empresaFactory->updateDsDirCert($cdinscesta, $cdempresa, $nrorg, $dsdircert);
    }

    public function getBairro($sequential, $country, $state, $county) {
        return $this->empresaFactory->getBairro($sequential, $country, $state, $county);
    }

    public function getBairroByName($country, $state, $county, $neighborhood) {
        return $this->empresaFactory->getBairroByName($country, $state, $county, $neighborhood);
    }

    public function criarDiretorio ($inscricao){
        try {
            $raiz = realpath(__DIR__ . "/../../../../../../../../"). DIRECTORY_SEPARATOR . "anexos" . DIRECTORY_SEPARATOR . "transmissaoNFe";
            $nrorgDir = $raiz . DIRECTORY_SEPARATOR . $inscricao['NRORG'];
            $ieDir    = $nrorgDir . DIRECTORY_SEPARATOR . $inscricao['CDINSCESTA'];
            $certDir = $ieDir . DIRECTORY_SEPARATOR . "cert";
            
            if (!is_dir($certDir)) {
                if (!mkdir($certDir, 0777, true)) {
                    throw new \Exception(
                        "Não foi possivel criar o diretorio $certDir. Verifique as permissões"
                    );
                }
            }
            return $certDir;
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function apagarArquivosDaPasta($dir){
        $dir = $dir.'\\';
        if(is_dir($dir)){
            if($handle = opendir($dir)){
                while(($file = readdir($handle)) !== false){
                    if($file != '.' && $file != '..'){
                        unlink($dir.$file);
                    }
                }
            }
        }
    }

    public function condicaoInscricao($row) {
        $conditions = array(
          array(
            "value" => $this->environment->getNrOrgTrab(),
            "operator" => "=",
            "columnName" => "NRORG"
          ),
          array(
            "value" => $row[0]['CDEMPRESA'],
            "operator" => "=",
            "columnName" => "CDEMPRESA"
          ),
          array(
            "value" => $row[0]['CDINSCESTA'],
            "operator" => "=",
            "columnName" => "CDINSCESTA"
          )
        );
        return $conditions;
    }

    public function realizaConsultaBanco($conditions,$dataSourceName) {
        $filterCriteria = new FilterCriteria($dataSourceName,$conditions);
        $dataSet = $this->buscar($filterCriteria);
        return $dataSet;
    }
    
    public function getArrayFiles($filesString, $getLink = false, $mode = 'prod') {
        $filesArray = array();
        $array = array_filter(explode('<', $filesString));
        $count = 0;
        foreach ($array as $element) {
            $element = explode('>', $element);
            $file = array("fileName" => $element[0]);
            if($getLink) $file["path"] = ($mode === 'prod') ? self::PROD_LINK.$element[1] : self::DEV_LINK.$element[1];
            array_push($filesArray, $file);
            $count++;
        }
        return $filesArray;
    }
    
      public function saveLogoEmpresa ($rowDsdirlogomarca){
        try {
            $file = $rowDsdirlogomarca['DSDIRLOGOMARCA'];
            $empresa = $rowDsdirlogomarca['CDEMPRESA'];
            $nrorg =  $this->environment->getNrorg();

       
            $fileName  = $file[0]['name'];
            $extension = explode(".", $fileName);
            $extension = end($extension);
           
            if(($extension !== self::EXTENSION_JPG) && ($extension !== self::EXTENSION_PNG)) {
                throw \Exception($fileName.': extensão do arquivo não aceita.');
            }
           else {
                $dsDirLogoEmp = '';
                $dsDirLogoEmp.='<'.$file[0]['name'].'>';
                $dsDirLogoEmp.=$file[0]["path"];
                $this->updateDsDirLogoEmpresa( $empresa, $nrorg, $dsDirLogoEmp, $file[0]['name']);
            }
            
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
     public function updateDsDirLogoEmpresa( $cdempresa, $nrorg, $dsdircert, $name) {
        $this->empresaFactory->updateDsDirLogoEmpresa( $cdempresa, $nrorg, $dsdircert,$name);
    }

}